package com.rubin.mybatis.builder;

import com.rubin.mybatis.io.Resource;
import com.rubin.mybatis.pojo.Configuration;
import com.rubin.mybatis.pojo.MapperStatement;
import com.rubin.mybatis.pojo.StatementType;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

/**
 * 解析mapper的构造器
 * Created by rubin on 2021/3/6.
 */
@Data
@AllArgsConstructor
public class MapperStatementBuilder extends AbstractBuilder {

    private Configuration configuration;

    /**
     * 解析配置文件 加载mapper到配置实体中
     *
     * @param configLocation
     * @throws DocumentException
     * @throws ClassNotFoundException
     */
    public void parse(String configLocation) throws DocumentException, ClassNotFoundException {
        Element root = new SAXReader().read(Resource.getResource(configLocation)).getRootElement();
        Element mapperScanElement = root.element("mapperScan");
        Properties properties = readElementAsProperties(mapperScanElement);
        parseList(properties.getProperty("base-package").replace(".", "/"));
    }

    /**
     * 解析mapper文件列表
     *
     * @param mapperBasePackage
     * @throws ClassNotFoundException
     * @throws DocumentException
     */
    public void parseList(String mapperBasePackage) throws ClassNotFoundException, DocumentException {
        List<File> mapperFiles = Resource.getResources(mapperBasePackage.replace(".", "/"));
        if (mapperFiles == null || mapperFiles.size() == 0) {
            throw new RuntimeException("the mapper xml file is not exist.");
        }
        parseList(mapperFiles);
    }

    /**
     * 根据mapper文件实体列表解析成配置实体列表
     *
     * @param mapperFiles
     * @throws DocumentException
     * @throws ClassNotFoundException
     */
    private void parseList(List<File> mapperFiles) throws DocumentException, ClassNotFoundException {
        for (File mapperFile : mapperFiles) {
            parseFile(mapperFile);
        }
    }

    /**
     * 解析mapper文件实体为配置实体
     *
     * @param mapperFile
     * @throws DocumentException
     * @throws ClassNotFoundException
     */
    private void parseFile(File mapperFile) throws DocumentException, ClassNotFoundException {
        Document document = new SAXReader().read(mapperFile);
        Element root = document.getRootElement();
        String namespace = root.attributeValue("namespace");
        for (StatementType statementType : StatementType.values()) {
            List<Element> statementElements = root.selectNodes("//" + statementType.getNode());
            if (statementElements == null || statementElements.size() == 0) {
                continue;
            }
            for (Element statementElement : statementElements) {
                parseStatementElement(statementElement, namespace, statementType);
            }
        }

    }

    /**
     * 解析select标签
     *
     * @param selectElement
     * @param namespace
     * @throws ClassNotFoundException
     */
    private void parseStatementElement(Element selectElement, String namespace, StatementType statementType) throws ClassNotFoundException {
        String id = selectElement.attributeValue("id"),
                sqlText = selectElement.getTextTrim();
        Class<?> parameterType = getClass(selectElement.attributeValue("parameterType")),
                resultType = getClass(selectElement.attributeValue("resultType"));
        MapperStatement mapperStatement = new MapperStatement();
        mapperStatement.setId(id);
        mapperStatement.setSql(sqlText);
        mapperStatement.setStatementType(statementType);
        mapperStatement.setParameterType(parameterType);
        mapperStatement.setResultType(resultType);
        if (configuration.getMapperStatementMap() == null) {
            configuration.setMapperStatementMap(new HashMap<>());
        }
        configuration.getMapperStatementMap().put(namespace + "." + id, mapperStatement);
    }

    /**
     * 获取配置的类类型
     *
     * @param className
     * @return
     * @throws ClassNotFoundException
     */
    private Class<?> getClass(String className) throws ClassNotFoundException {
        if (className == null) {
            return null;
        }
        return Class.forName(className);
    }

}
