package com.rubin.mybatis.executor;

import com.rubin.mybatis.connection.ConnectionFactory;
import com.rubin.mybatis.pojo.BoundSql;
import com.rubin.mybatis.pojo.MapperStatement;
import com.rubin.mybatis.pojo.ParameterMapping;
import com.rubin.mybatis.utils.GenericTokenParser;
import com.rubin.mybatis.utils.ParameterMappingTokenHandler;
import lombok.AllArgsConstructor;
import lombok.Data;

import java.lang.reflect.Field;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * 默认的执行器
 * Created by rubin on 2021/3/7.
 */
@Data
@AllArgsConstructor
public class DefaultExecutor implements Executor {

    private ConnectionFactory connectionFactory;

    /**
     * 执行查询操作
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws SQLException
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    @Override
    public <E> List<E> query(MapperStatement mapperStatement, Object arg) throws SQLException, NoSuchFieldException, IllegalAccessException, InstantiationException {
        BoundSql boundSql = parseCustomSql(mapperStatement.getSql());
        final PreparedStatement preparedStatement = connectionFactory.getConnection().prepareStatement(boundSql.getSql());
        setParam(boundSql, mapperStatement, preparedStatement, arg);
        ResultSet resultSet = preparedStatement.executeQuery();
        List<Object> resultList = parseQueryResult(resultSet, mapperStatement);
        if (resultList == null) {
            return new ArrayList<>();
        }
        return (List<E>) resultList;
    }

    /**
     * 更新单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    @Override
    public int update(MapperStatement mapperStatement, Object arg) throws Exception {
        BoundSql boundSql = parseCustomSql(mapperStatement.getSql());
        final PreparedStatement preparedStatement = connectionFactory.getConnection().prepareStatement(boundSql.getSql());
        setParam(boundSql, mapperStatement, preparedStatement, arg);
        return preparedStatement.executeUpdate();
    }

    /**
     * 关闭连接
     *
     * @throws SQLException
     */
    @Override
    public void close() throws SQLException {
        connectionFactory.getConnection().close();
    }

    /**
     * 设置参数
     *
     * @param boundSql
     * @param mapperStatement
     * @param preparedStatement
     * @param arg
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     * @throws SQLException
     */
    private void setParam(BoundSql boundSql, MapperStatement mapperStatement, PreparedStatement preparedStatement, Object arg) throws NoSuchFieldException, IllegalAccessException, SQLException {
        List<ParameterMapping> parameterMappings = boundSql.getParameterMappings();
        if (parameterMappings != null && parameterMappings.size() > 0) {
            if (arg == null) {
                throw new RuntimeException("arg can not be null.");
            }
            for (int i = 0, iLength = parameterMappings.size(); i < iLength; i++) {
                String paramName = parameterMappings.get(i).getContent();
                Field field = mapperStatement.getParameterType().getDeclaredField(paramName);
                field.setAccessible(true);
                Object fieldValue = field.get(arg);
                preparedStatement.setObject(i + 1, fieldValue);
            }
        }
    }

    /**
     * 解析查询结果
     *
     * @param resultSet
     * @param mapperStatement
     * @return
     * @throws SQLException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchFieldException
     */
    private List<Object> parseQueryResult(ResultSet resultSet, MapperStatement mapperStatement) throws SQLException, IllegalAccessException, InstantiationException, NoSuchFieldException {
        List<Object> results = new ArrayList<>();
        while (resultSet.next()) {
            ResultSetMetaData metaData = resultSet.getMetaData();
            Class resultType = mapperStatement.getResultType();
            Object resultItem = resultType.newInstance();
            for (int i = 1, iLength = metaData.getColumnCount(); i <= iLength; i++) {
                // 此处使用metaData.getColumnLabel(i)解决别名问题 课程中未考虑此问题
                String columnName = metaData.getColumnLabel(i);
                Field field = resultType.getDeclaredField(columnName);
                field.setAccessible(true);
                Object fieldValue = resultSet.getObject(i);
                field.set(resultItem, fieldValue);
            }
            results.add(resultItem);
        }
        return results;
    }

    /**
     * 解析用户的自定义sql为jdbc标准sql 并顺序封装占位参数名称
     *
     * @param sql
     * @return
     */
    private BoundSql parseCustomSql(String sql) {
        ParameterMappingTokenHandler parameterMappingTokenHandler = new ParameterMappingTokenHandler();
        GenericTokenParser genericTokenParser = new GenericTokenParser("#{", "}", parameterMappingTokenHandler);
        String parseSql = genericTokenParser.parse(sql);
        BoundSql boundSql = new BoundSql(parseSql, parameterMappingTokenHandler.getParameterMappings());
        return boundSql;
    }

}
