package com.rubin.mybatis.sqlsession;

import com.rubin.mybatis.connection.ConnectionFactory;
import com.rubin.mybatis.executor.DefaultExecutor;
import com.rubin.mybatis.executor.Executor;
import com.rubin.mybatis.pojo.Configuration;
import com.rubin.mybatis.pojo.MapperStatement;
import lombok.AllArgsConstructor;
import lombok.Data;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.sql.SQLException;
import java.util.List;

/**
 * SqlSession默认实现类
 * Created by rubin on 2021/3/6.
 */
@Data
@AllArgsConstructor
public class DefaultSqlSession implements SqlSession {

    private Configuration configuration;

    private Executor executor;

    public DefaultSqlSession(Configuration configuration, ConnectionFactory connectionFactory) throws SQLException {
        this.configuration = configuration;
        this.executor = new DefaultExecutor(connectionFactory);
    }

    /**
     * 查询列表
     * 此处和课程不同 是因为课程中的可变参数后面并未维护，而是硬编码取第一个
     * 个人认为此种方式不太好，要么就在源头限制方法作用，要么就实现可变参的支持
     * 纯个人意见
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws SQLException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws NoSuchFieldException
     */
    @Override
    public <E> List<E> selectList(MapperStatement mapperStatement, Object arg) throws Exception {
        List<E> results = executor.query(mapperStatement, arg);
        return results;
    }

    /**
     * 查询单条
     * 此处和课程不同 是因为课程中的可变参数后面并未维护，而是硬编码取第一个
     * 个人认为此种方式不太好，要么就在源头限制方法作用，要么就实现可变参的支持
     * 纯个人意见
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws SQLException
     * @throws NoSuchFieldException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    @Override
    public <T> T selectOne(MapperStatement mapperStatement, Object arg) throws Exception {
        List<Object> objects = selectList(mapperStatement, arg);
        if (objects == null) {
            return null;
        }
        if (objects.size() == 1) {
            return (T) objects.get(0);
        }
        throw new RuntimeException("there are too many result.");
    }

    /**
     * 插入单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    @Override
    public int insert(MapperStatement mapperStatement, Object arg) throws Exception {
        return executor.update(mapperStatement, arg);
    }

    /**
     * 更新单条记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    @Override
    public int update(MapperStatement mapperStatement, Object arg) throws Exception {
        return executor.update(mapperStatement, arg);
    }

    /**
     * 删除单挑记录
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    @Override
    public int delete(MapperStatement mapperStatement, Object arg) throws Exception {
        return executor.update(mapperStatement, arg);
    }

    /**
     * 获取mapper接口的代理类
     *
     * @param mapperClass
     * @param <T>
     * @return
     */
    @Override
    public <T> T getMapper(Class<T> mapperClass) {
        T t = (T) Proxy.newProxyInstance(DefaultSqlSession.class.getClassLoader(), new Class[]{mapperClass}, (proxy, method, args) -> {
            String methodName = method.getName(),
                    className = method.getDeclaringClass().getName(),
                    id = className + "." + methodName;
            MapperStatement mapperStatement = configuration.getMapperStatementMap().get(id);
            Object arg = null;
            if (args != null && args.length > 0) {
                arg = args[0];
            }
            switch (mapperStatement.getStatementType()) {
                case SELECT:
                    return handleSelect(method, mapperStatement, arg);
                case INSERT:
                    return handleInsert(mapperStatement, arg);
                case UPDATE:
                    return handleUpdate(mapperStatement, arg);
                case DELETE:
                    return handleDelete(mapperStatement, arg);
                default:
                    return null;
            }

        });
        return t;
    }

    /**
     * 处理查询操作
     *
     * @param method
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    private Object handleSelect(Method method, MapperStatement mapperStatement, Object arg) throws Exception {
        Type returnType = method.getGenericReturnType();
        if (returnType instanceof ParameterizedType) {
            return selectList(mapperStatement, arg);
        }
        return selectOne(mapperStatement, arg);
    }

    /**
     * 处理插入操作
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    private Object handleInsert(MapperStatement mapperStatement, Object arg) throws Exception {
        return insert(mapperStatement, arg);
    }

    /**
     * 处理更新操作
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    private Object handleUpdate(MapperStatement mapperStatement, Object arg) throws Exception {
        return update(mapperStatement, arg);
    }

    /**
     * 处理删除操作
     *
     * @param mapperStatement
     * @param arg
     * @return
     * @throws Exception
     */
    private Object handleDelete(MapperStatement mapperStatement, Object arg) throws Exception {
        return delete(mapperStatement, arg);
    }

    /**
     * 关闭连接
     *
     * @throws SQLException
     */
    @Override
    public void close() throws SQLException {
        executor.close();
    }

}
