package com.rubin.spring.context;

import com.rubin.spring.context.bean.BeanDefinitionLoader;
import com.rubin.spring.context.factory.AutowiredBeanFactory;
import com.rubin.spring.context.factory.DefaultAutowiredBeanFactory;
import com.rubin.spring.context.processor.BeanFactoryPostProcessor;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ReflectUtil;

import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

public abstract class AbstractApplicationContext implements ApplicationContext {

    /**
     * 容器对象
     */
    private AutowiredBeanFactory autowiredBeanFactory;

    /**
     * bean定义加载器
     */
    private BeanDefinitionLoader beanDefinitionLoader;

    public AbstractApplicationContext(BeanDefinitionLoader beanDefinitionLoader) {
        this.beanDefinitionLoader = beanDefinitionLoader;
        refresh();
    }

    @Override
    public void refresh() {

        AutowiredBeanFactory autowiredBeanFactory = getAutowiredBeanFactory();

        // 准备BeanFactory
        prepareBeanFactory(autowiredBeanFactory);

        // 初始化容器对象之后的逻辑
        postBeanFactory(autowiredBeanFactory);

        // 留给子类实现
        onRefresh();

        // 结束刷新
        finishRefresh();

    }

    /**
     * 刷新时的扩展方法
     */
    protected abstract void onRefresh();


    /**
     * 准备BeanFactory
     *
     * @param autowiredBeanFactory
     */
    private void prepareBeanFactory(AutowiredBeanFactory autowiredBeanFactory) {
        this.autowiredBeanFactory = autowiredBeanFactory;
    }

    /**
     * 初始化容器对象之后的逻辑
     *
     * @param autowiredBeanFactory
     */
    private void postBeanFactory(AutowiredBeanFactory autowiredBeanFactory) {
        // 执行所有的BeanFactoryPostProcessor实现类自定义逻辑
        triggerAllBeanFactoryPostProcessors(autowiredBeanFactory);
    }

    /**
     * 触发所有的BeanFactoryPostProcessor
     *
     * @param autowiredBeanFactory
     */
    private void triggerAllBeanFactoryPostProcessors(AutowiredBeanFactory autowiredBeanFactory) {
        Set<Class> beanFactoryPostProcessorClasses = ReflectUtil.getAllSubTypeOf(BeanFactoryPostProcessor.class);
        List<BeanFactoryPostProcessor> beanFactoryPostProcessors = beanFactoryPostProcessorClasses.stream()
                .map(ReflectUtil::createInstance)
                .map(o -> (BeanFactoryPostProcessor) o)
                .sorted(Comparator.comparing(BeanFactoryPostProcessor::getOrder))
                .collect(Collectors.toList());
        if (CollectionUtil.isEmpty(beanFactoryPostProcessors)) {
            return;
        }
        for (BeanFactoryPostProcessor beanFactoryPostProcessor : beanFactoryPostProcessors) {
            beanFactoryPostProcessor.postProcessBeanFactory(autowiredBeanFactory);
        }
    }

    /**
     * 初始化结束
     */
    private void finishRefresh() {
        autowiredBeanFactory.preInstantiateSingletons();
    }

    /**
     * 获取AutowiredBeanFactory
     *
     * @return
     */
    private AutowiredBeanFactory getAutowiredBeanFactory() {
        if (autowiredBeanFactory != null) {
            return autowiredBeanFactory;
        }
        return new DefaultAutowiredBeanFactory(this.beanDefinitionLoader);
    }

    /**
     * 通过name获取bean
     *
     * @param beanName
     * @return
     */
    @Override
    public Object getBean(String beanName) {
        return autowiredBeanFactory.getBean(beanName);
    }

    /**
     * 通过类型获取bean
     *
     * @param beanClass
     * @return
     */
    @Override
    public Object getBean(Class<?> beanClass) {
        return autowiredBeanFactory.getBean(beanClass);
    }

    /**
     * 包含某个bean
     *
     * @param beanName
     * @return
     */
    @Override
    public boolean containsBean(String beanName) {
        return autowiredBeanFactory.containsBean(beanName);
    }

}
