package com.rubin.spring.context.bean;

import com.rubin.spring.context.annotation.Autowired;
import com.rubin.spring.context.annotation.Component;
import com.rubin.spring.context.annotation.Value;
import com.rubin.spring.context.utils.BeanUtil;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ReflectUtil;
import com.rubin.spring.context.utils.StringUtil;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 注解bean定义加载器
 */
public class AnnotationBeanDefinitionLoader implements BeanDefinitionLoader {

    private String annotationScanBasePackage;

    private Map<String, BeanDefinition> beanDefinitionMap;

    public AnnotationBeanDefinitionLoader(String annotationScanBasePackage) {
        this.annotationScanBasePackage = annotationScanBasePackage;
        this.beanDefinitionMap = new ConcurrentHashMap<>();
    }

    @Override
    public Map<String, BeanDefinition> load() {
        Set<Class> beanClasses = ReflectUtil.getAllObjectClassWithAnnotationOrSubAnnotation(annotationScanBasePackage, Component.class);
        if (!CollectionUtil.isEmpty(beanClasses)) {
            beanClasses.stream().forEach(beanClass -> parseBeanClass(beanClass));
        }
        return beanDefinitionMap;
    }

    /**
     * 解析beanClass为BeanDefinition
     *
     * @param beanClass
     */
    private void parseBeanClass(Class beanClass) {
        String beanName = getBeanName(beanClass);
        BeanDefinition beanDefinition = new BeanDefinition();
        beanDefinition.setBeanName(beanName);
        beanDefinition.setBeanClass(beanClass);
        beanDefinition.setInterfaces(ReflectUtil.getInterfaces(beanClass));
        beanDefinition.setInitMethodName(StringUtil.EMPTY);
        beanDefinition.setFieldDefinitions(new ArrayList<>());
        Field[] declaredFields = beanClass.getDeclaredFields();
        for (int i = 0; i < declaredFields.length; i++) {
            parseBeanField(beanDefinition, declaredFields[i]);
        }
        beanDefinitionMap.put(beanName, beanDefinition);
    }

    /**
     * 解析依赖注入字段
     *
     * @param beanDefinition
     * @param declaredField
     */
    private void parseBeanField(BeanDefinition beanDefinition, Field declaredField) {
        if (!declaredField.isAnnotationPresent(Autowired.class) && !declaredField.isAnnotationPresent(Value.class)) {
            return;
        }
        FieldDefinition fieldDefinition = new FieldDefinition();
        fieldDefinition.setFieldName(declaredField.getName());
        fieldDefinition.setFieldClass(declaredField.getType());
        if (declaredField.isAnnotationPresent(Autowired.class)) {
            fieldDefinition.setAutowired(true);
            fieldDefinition.setValue(StringUtil.EMPTY);
        } else {
            fieldDefinition.setAutowired(false);
            Value valueAnnotation = declaredField.getAnnotation(Value.class);
            String value = valueAnnotation.value();
            if (StringUtil.isBlank(value)) {
                throw new RuntimeException("the @Value annotation can not be blank");
            }
            fieldDefinition.setValue(value);
        }
        fieldDefinition.setInjectType(InjectType.FIELD);
        fieldDefinition.setRef(StringUtil.EMPTY);
        beanDefinition.getFieldDefinitions().add(fieldDefinition);
    }

    /**
     * 获取bean的名称
     *
     * @param beanClass
     * @return
     */
    private String getBeanName(Class<?> beanClass) {
        if (beanClass.isAnnotationPresent(Component.class)) {
            Component component = beanClass.getAnnotation(Component.class);
            if (StringUtil.isNotBlank(component.value())) {
                return component.value();
            }
        }
        Annotation subAnnotation = ReflectUtil.getSubAnnotation(beanClass, Component.class);
        Method[] declaredMethods = subAnnotation.getClass().getDeclaredMethods();
        for (int i = 0; i < declaredMethods.length; i++) {
            if (declaredMethods[i].getName().equals("value")) {
                try {
                    String beanName = String.valueOf(declaredMethods[i].invoke(subAnnotation, new Object[]{}));
                    if (StringUtil.isNotBlank(beanName)) {
                        return beanName;
                    }
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                }
            }
        }
        return BeanUtil.getDefaultBeanName(beanClass.getSimpleName());
    }

}
