package com.rubin.spring.context.bean;

import com.rubin.spring.context.io.Resources;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ReflectUtil;
import com.rubin.spring.context.utils.StringUtil;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * xml文件bean定义加载器
 */
public class ClassPathXmlBeanDefinitionLoader implements BeanDefinitionLoader {

    private String configLocation;

    private Map<String, BeanDefinition> beanDefinitionMap;

    public ClassPathXmlBeanDefinitionLoader(String configLocation) {
        this.configLocation = configLocation;
        this.beanDefinitionMap = new ConcurrentHashMap<>();
    }

    @Override
    public Map<String, BeanDefinition> load() {
        // 解析xml
        SAXReader saxReader = new SAXReader();
        Document document = null;
        try {
            document = saxReader.read(Resources.readFileAsInputStream(configLocation));
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        Element rootElement = document.getRootElement();
        loadXmlBeanDefinitions(rootElement);
        String annotationScanBasePackage = checkAnnotationScanOpenAndGetBasePackage(rootElement);
        if (StringUtil.isNotBlank(annotationScanBasePackage)) {
            loadAndMergeAnnotationBeanDefinitions(annotationScanBasePackage);
        }
        return beanDefinitionMap;
    }

    /**
     * 加载并合并bean定义映射
     *
     * @param annotationScanBasePackage
     */
    private void loadAndMergeAnnotationBeanDefinitions(String annotationScanBasePackage) {
        Map<String, BeanDefinition> annotationBeanDefinitionMap = new AnnotationBeanDefinitionLoader(annotationScanBasePackage).load();
        if (annotationBeanDefinitionMap.size() > 0) {
            for (Map.Entry<String, BeanDefinition> annotationBeanDefinitionEntry : annotationBeanDefinitionMap.entrySet()) {
                if (beanDefinitionMap.containsKey(annotationBeanDefinitionEntry.getKey())) {
                    throw new RuntimeException("the bean id " + annotationBeanDefinitionEntry.getKey() + "have been registered");
                }
                beanDefinitionMap.put(annotationBeanDefinitionEntry.getKey(), annotationBeanDefinitionEntry.getValue());
            }
        }
    }

    /**
     * 加载xml中定义的bean
     *
     * @param rootElement
     */
    private void loadXmlBeanDefinitions(Element rootElement) {
        List<Element> beanElementList = rootElement.selectNodes("//bean");
        if (CollectionUtil.isEmpty(beanElementList)) {
            return;
        }
        beanElementList.stream().forEach(beanElement -> parseBeanElement(beanElement));
    }

    /**
     * 解析bean标签
     *
     * @param beanElement
     */
    private void parseBeanElement(Element beanElement) {
        String beanName = beanElement.attributeValue("id"),
                classPath = beanElement.attributeValue("class"),
                initMethod = beanElement.attributeValue("init-method");
        Class beanClass = ReflectUtil.getClass(classPath);
        List<Class> interfaces = ReflectUtil.getInterfaces(beanClass);
        if (StringUtil.isAnyBlank(beanName, classPath)) {
            throw new RuntimeException("the attribute id and class can not be empty");
        }
        BeanDefinition beanDefinition = new BeanDefinition();
        beanDefinition.setBeanName(beanName);
        beanDefinition.setBeanClass(beanClass);
        beanDefinition.setInterfaces(interfaces);
        beanDefinition.setInitMethodName(initMethod);
        List<Element> propertyElements = beanElement.selectNodes("property");
        parseBeanPropertyElements(beanDefinition, propertyElements);
        beanDefinitionMap.put(beanName, beanDefinition);
    }

    /**
     * 解析bean属性标签列表
     *
     * @param beanDefinition
     * @param propertyElements
     */
    private void parseBeanPropertyElements(BeanDefinition beanDefinition, List<Element> propertyElements) {
        if (CollectionUtil.isEmpty(propertyElements)) {
            beanDefinition.setFieldDefinitions(new ArrayList<>());
            return;
        }
        List<FieldDefinition> fieldDefinitionList = propertyElements.stream().map(propertyElement -> parseBeanPropertyElement(beanDefinition, propertyElement)).collect(Collectors.toList());
        beanDefinition.setFieldDefinitions(fieldDefinitionList);
    }

    /**
     * 解析bean属性标签
     *
     * @param beanDefinition
     * @param propertyElement
     */
    private FieldDefinition parseBeanPropertyElement(BeanDefinition beanDefinition, Element propertyElement) {
        String fieldName = propertyElement.attributeValue("name"),
                ref = propertyElement.attributeValue("ref"),
                value = propertyElement.attributeValue("value");
        if (StringUtil.isNoneBlank(ref, value)) {
            throw new RuntimeException("the property attribute ref and value can not coexist");
        }
        FieldDefinition fieldDefinition = new FieldDefinition();
        fieldDefinition.setFieldName(fieldName);
        fieldDefinition.setRef(ref);
        fieldDefinition.setValue(value);
        fieldDefinition.setFieldClass(getFieldClass(beanDefinition.getBeanClass(), fieldName));
        fieldDefinition.setInjectType(getInjectType(beanDefinition.getBeanClass(), fieldName));
        fieldDefinition.setAutowired(false);
        return fieldDefinition;
    }

    /**
     * 获取属性类型
     *
     * @param beanClass
     * @param fieldName
     * @return
     */
    private Class getFieldClass(Class beanClass, String fieldName) {
        try {
            Field field = beanClass.getField(fieldName);
            return field.getType();
        } catch (NoSuchFieldException e) {
            Method[] declaredMethods = beanClass.getDeclaredMethods();
            for (int i = 0; i < declaredMethods.length; i++) {
                if (declaredMethods[i].getName().equalsIgnoreCase("set" + fieldName)) {
                    Class<?>[] parameterTypes = declaredMethods[i].getParameterTypes();
                    if (parameterTypes == null || parameterTypes.length == 0) {
                        throw new RuntimeException("there are no field or set method matched " + fieldName);
                    }
                    return parameterTypes[0];
                }
            }
        }
        throw new RuntimeException("there are no field or set method matched " + fieldName);
    }

    /**
     * 获取注入类型
     *
     * @param beanClass
     * @param fieldName
     * @return
     */
    private InjectType getInjectType(Class beanClass, String fieldName) {
        try {
            if (beanClass.getField(fieldName) != null) {
                return InjectType.FIELD;
            }
            return InjectType.SET;
        } catch (NoSuchFieldException e) {
            return InjectType.SET;
        }
    }

    /**
     * 查看是否开启了注解扫描
     *
     * @param rootElement
     * @return
     */
    public String checkAnnotationScanOpenAndGetBasePackage(Element rootElement) {
        List<Element> annotationElements = rootElement.selectNodes("//component-scan");
        if (CollectionUtil.isEmpty(annotationElements)) {
            return null;
        }
        if (annotationElements.size() > 1) {
            throw new RuntimeException("the annotation-scan element only can defined for once");
        }
        String basePackage = annotationElements.get(0).attributeValue("base-package");
        if (StringUtil.isBlank(basePackage)) {
            throw new RuntimeException("the base-package can not be blank");
        }
        return basePackage;
    }

}
