package com.rubin.spring.context.factory;

import com.rubin.spring.context.bean.BeanDefinition;
import com.rubin.spring.context.bean.BeanDefinitionLoader;
import com.rubin.spring.context.bean.FieldDefinition;
import com.rubin.spring.context.processor.BeanPostProcessor;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ProxyUtil;
import com.rubin.spring.context.utils.ReflectUtil;
import com.rubin.spring.context.utils.StringUtil;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public class DefaultAutowiredBeanFactory implements AutowiredBeanFactory {

    /**
     * 一级缓存 即单例池
     */
    private Map<String, Object> singletonObjects;

    /**
     * 二级缓存 存放提前暴露的对象实体，解决循环依赖问题
     */
    private Map<String, Object> objectMappings;

    /**
     * bean的id和定义实体的映射表
     */
    private Map<String, BeanDefinition> beanDefinitionMap;

    /**
     * bean定义加载器
     */
    private BeanDefinitionLoader beanDefinitionLoader;

    public DefaultAutowiredBeanFactory(BeanDefinitionLoader beanDefinitionLoader) {
        this.singletonObjects = new ConcurrentHashMap<>();
        this.objectMappings = new ConcurrentHashMap<>();
        this.beanDefinitionMap = new ConcurrentHashMap<>();
        this.beanDefinitionLoader = beanDefinitionLoader;
        this.beanDefinitionMap.putAll(this.beanDefinitionLoader.load());
    }

    /**
     * 通过name获取bean
     *
     * @param beanName
     * @return
     */
    @Override
    public Object getBean(String beanName) {
        Object bean = singletonObjects.get(beanName);
        if (bean == null) {
            BeanDefinition beanDefinition = getBeanDefinitionByName(beanName);
            bean = createBean(beanDefinition);
        }
        return bean;
    }

    /**
     * 通过类型获取bean
     * 同一类型只支持一个bean存在 如果有多个bean将报错 接口同理
     *
     * @param beanClass
     * @return
     */
    @Override
    public Object getBean(Class<?> beanClass) {
        Object bean;
        // 接口类型  获取其子实现类 多个子实现类将报错
        if (beanClass.isInterface()) {
            bean = checkAndGetImplObject(beanClass);
        } else {
            bean = checkAndGetObject(beanClass);
        }
        if (bean == null) {
            BeanDefinition beanDefinition = getBeanDefinitionByClass(beanClass);
            bean = createBean(beanDefinition);
        }
        return bean;
    }

    /**
     * 包含某个bean
     *
     * @param beanName
     * @return
     */
    @Override
    public boolean containsBean(String beanName) {
        return singletonObjects.containsKey(beanName);
    }

    /**
     * 注册Bean对象
     *
     * @param beanName
     * @param beanObject
     */
    @Override
    public synchronized void registerBean(String beanName, Object beanObject) {
        beanObject = triggerAllBeanPostProcessors(beanName, beanObject);
        if (containsBean(beanName)) {
            throw new RuntimeException("there are same bean named " + beanName);
        }
        singletonObjects.put(beanName, beanObject);
    }

    /**
     * 准备实例化所有单例对象
     */
    @Override
    public void preInstantiateSingletons() {
        if (beanDefinitionMap.size() == 0) {
            return;
        }
        for (BeanDefinition beanDefinition : beanDefinitionMap.values()) {
            createBean(beanDefinition);
        }
    }

    /**
     * 创建bean实体并装配 装配完毕自动注入容器中
     *
     * @param beanDefinition
     * @return
     */
    private Object createBean(BeanDefinition beanDefinition) {
        Object beanObject;
        if ((beanObject = singletonObjects.get(beanDefinition.getBeanName())) != null) {
            return beanObject;
        }
        if ((beanObject = objectMappings.get(beanDefinition.getBeanName())) != null) {
            return beanObject;
        }
        beanObject = doCreateBean(beanDefinition);
        return beanObject;
    }

    /**
     * 执行实体创建
     *
     * @param beanDefinition
     * @return
     */
    private Object doCreateBean(BeanDefinition beanDefinition) {
        if (beanDefinition.getBeanClass().isInterface()) {
            throw new RuntimeException("the interface can not be instanced");
        }
        Object beanObject = ReflectUtil.createInstance(beanDefinition.getBeanClass());
        // 放入二级缓存
        objectMappings.put(beanDefinition.getBeanName(), beanObject);
        fillProperties(beanObject, beanDefinition.getBeanClass(), beanDefinition.getFieldDefinitions());
        invokeInitMethod(beanDefinition, beanObject);
        // 属性填充完毕 移除二级缓存 放入一级缓存
        objectMappings.remove(beanDefinition.getBeanName());
        registerBean(beanDefinition.getBeanName(), beanObject);
        return beanObject;
    }

    /**
     * 执行init-method
     *
     * @param beanDefinition
     * @param beanObject
     */
    private void invokeInitMethod(BeanDefinition beanDefinition, Object beanObject) {
        if (StringUtil.isNotBlank(beanDefinition.getInitMethodName())) {
            Method[] declaredMethods = beanObject.getClass().getDeclaredMethods();
            for (int i = 0; i < declaredMethods.length; i++) {
                if (declaredMethods[i].getName().equals(beanDefinition.getInitMethodName())) {
                    try {
                        declaredMethods[i].invoke(beanObject, new Object[]{});
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    } catch (InvocationTargetException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * 填充实体所有定义的属性
     *
     * @param beanObject
     * @param beanClass
     * @param fieldDefinitions
     */
    private void fillProperties(Object beanObject, Class beanClass, List<FieldDefinition> fieldDefinitions) {
        if (CollectionUtil.isEmpty(fieldDefinitions)) {
            return;
        }
        for (FieldDefinition fieldDefinition : fieldDefinitions) {
            doFillProperty(beanObject, fieldDefinition);
        }
    }

    /**
     * 执行填充属性
     *
     * @param beanObject
     * @param fieldDefinition
     */
    private void doFillProperty(Object beanObject, FieldDefinition fieldDefinition) {
        Object fieldValue;
        if (fieldDefinition.isAutoWired()) {
            fieldValue = getBean(fieldDefinition.getFieldClass());
            if (fieldValue == null) {
                if (fieldDefinition.getFieldClass().isInterface()) {
                    fieldValue = createBean(getBeanDefinitionByInterfaceClass(fieldDefinition.getFieldClass()));
                } else {
                    fieldValue = createBean(getBeanDefinitionByClass(fieldDefinition.getFieldClass()));
                }
            }
        } else {
            if (fieldDefinition.isRef()) {
                fieldValue = createBean(beanDefinitionMap.get(fieldDefinition.getRef()));
            } else {
                fieldValue = handleFieldValue(fieldDefinition.getFieldClass(), fieldDefinition.getValue());
            }
        }
        if (fieldDefinition.isSetInject()) {
            ReflectUtil.setFieldValue(beanObject, fieldDefinition.getFieldName(), fieldValue);
            return;
        }
        ReflectUtil.injectFieldValue(beanObject, fieldDefinition.getFieldName(), fieldValue);
    }

    /**
     * 根据接口类型查找定义的bean定义实体
     *
     * @param fieldClass
     * @return
     */
    private BeanDefinition getBeanDefinitionByInterfaceClass(Class fieldClass) {
        List<BeanDefinition> hitBeanDefinitions = beanDefinitionMap.values().stream().filter(beanDefinition -> beanDefinition.getInterfaces().contains(fieldClass)).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(hitBeanDefinitions)) {
            throw new RuntimeException("there are no bean match the class " + fieldClass.getCanonicalName());
        }
        if (hitBeanDefinitions.size() > 1) {
            throw new RuntimeException("there are one more bean match the class " + fieldClass.getCanonicalName());
        }
        return hitBeanDefinitions.get(0);
    }

    /**
     * 通过bean名称获取bean定义
     *
     * @param beanName
     * @return
     */
    private BeanDefinition getBeanDefinitionByName(String beanName) {
        BeanDefinition beanDefinition = beanDefinitionMap.get(beanName);
        if (beanDefinition == null) {
            throw new RuntimeException("there are no bean named " + beanName);
        }
        return beanDefinition;
    }

    /**
     * 根据类型查找定义的bean定义实体
     *
     * @param fieldClass
     * @return
     */
    private BeanDefinition getBeanDefinitionByClass(Class fieldClass) {
        List<BeanDefinition> hitBeanDefinitions = beanDefinitionMap.values().stream()
                .filter(beanDefinition -> beanDefinition.getBeanClass().equals(fieldClass) || beanDefinition.getInterfaces().contains(fieldClass))
                .collect(Collectors.toList());
        if (CollectionUtil.isEmpty(hitBeanDefinitions)) {
            throw new RuntimeException("there are no bean match the class " + fieldClass.getCanonicalName());
        }
        if (hitBeanDefinitions.size() > 1) {
            throw new RuntimeException("there are one more bean match the class " + fieldClass.getCanonicalName());
        }
        return hitBeanDefinitions.get(0);
    }

    /**
     * 处理基本类型
     *
     * @param paramType
     * @param value
     * @return
     */
    private Object handleFieldValue(Class paramType, String value) {
        if (byte.class.equals(paramType)) {
            return Byte.valueOf(value).byteValue();
        } else if (short.class.equals(paramType)) {
            return Short.valueOf(value).shortValue();
        } else if (int.class.equals(paramType)) {
            return Integer.valueOf(value).intValue();
        } else if (long.class.equals(paramType)) {
            return Long.valueOf(value).longValue();
        } else if (double.class.equals(paramType)) {
            return Double.valueOf(value).doubleValue();
        } else if (float.class.equals(paramType)) {
            return Float.valueOf(value).floatValue();
        } else if (char.class.equals(paramType)) {
            return Character.valueOf(value.toCharArray()[0]).charValue();
        } else if (boolean.class.equals(paramType)) {
            return Boolean.valueOf(value).booleanValue();
        } else if (Byte.class.equals(paramType)) {
            return Byte.valueOf(value);
        } else if (Short.class.equals(paramType)) {
            return Short.valueOf(value);
        } else if (Integer.class.equals(paramType)) {
            return Integer.valueOf(value);
        } else if (Long.class.equals(paramType)) {
            return Long.valueOf(value);
        } else if (Double.class.equals(paramType)) {
            return Double.valueOf(value);
        } else if (Float.class.equals(paramType)) {
            return Float.valueOf(value);
        } else if (Character.class.equals(paramType)) {
            return Character.valueOf(value.toCharArray()[0]);
        } else if (Boolean.class.equals(paramType)) {
            return Boolean.valueOf(value);
        } else if (String.class.equals(paramType)) {
            return value;
        }
        return null;
    }

    /**
     * 触发所有的BeanPostProcessor执行逻辑
     *
     * @param beanName
     * @param beanObject
     * @return
     */
    private Object triggerAllBeanPostProcessors(String beanName, Object beanObject) {
        Set<Class> beanPostProcessorClasses = ReflectUtil.getAllSubTypeOf(BeanPostProcessor.class);
        List<BeanPostProcessor> beanPostProcessors = beanPostProcessorClasses.stream()
                .map(ReflectUtil::createInstance)
                .map(o -> (BeanPostProcessor) o)
                .sorted(Comparator.comparing(BeanPostProcessor::getOrder))
                .collect(Collectors.toList());
        if (CollectionUtil.isEmpty(beanPostProcessors)) {
            return beanObject;
        }
        for (BeanPostProcessor beanPostProcessor : beanPostProcessors) {
            beanObject = beanPostProcessor.postProcessBeforeRegister(this, beanName, beanObject);
        }
        return beanObject;
    }

    /**
     * 检查该接口的实现类个数并返回该实现类实体
     * 注意：只可以判断直接实现该接口的类  间接实现的情况未考虑
     *
     * @param interfaceClass
     * @return
     */
    private Object checkAndGetImplObject(Class<?> interfaceClass) {
        int matchCount = 0;
        Object bean = null;
        for (Object value : singletonObjects.values()) {
            Class beanClass = checkProxyAndGetTarget(value);
            if (beanClass.equals(interfaceClass)) {
                bean = value;
                matchCount++;
            } else if (Arrays.asList(value.getClass().getInterfaces()).contains(interfaceClass)) {
                bean = value;
                matchCount++;
            }
        }
        if (matchCount > 1) {
            throw new RuntimeException("there are more than one object match the class");
        }
        return bean;
    }

    /**
     * 检查并返回匹配类型的实体
     *
     * @param beanClass
     * @return
     */
    private Object checkAndGetObject(Class<?> beanClass) {
        int matchCount = 0;
        Object bean = null;
        for (Object value : singletonObjects.values()) {
            Class targetClass = checkProxyAndGetTarget(value);
            if (beanClass.equals(targetClass)) {
                bean = value;
                matchCount++;
            }
        }
        if (matchCount > 1) {
            throw new RuntimeException("there are more than one object match the class");
        }
        return bean;
    }

    /**
     * 检查是否是代理对象 是的话 获取代理目标类
     *
     * @param value
     * @return
     */
    private Class checkProxyAndGetTarget(Object value) {
        if (ProxyUtil.checkIsJdkProxy(value)) {
            try {
                return ProxyUtil.getJdkProxyTarget(value);
            } catch (NoSuchFieldException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        if (ProxyUtil.checkIsCGlibProxy(value)) {
            try {
                return ProxyUtil.getCGlibProxyTarget(value);
            } catch (NoSuchFieldException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return value.getClass();
    }

}
