package com.rubin.spring.context.utils;

import com.rubin.spring.context.handler.JdkProxyHandler;
import com.rubin.spring.context.interceptor.CGlibProxyMethodInterceptor;
import net.sf.cglib.proxy.Enhancer;
import net.sf.cglib.proxy.MethodInterceptor;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;

/**
 * 代理工具类
 */
public class ProxyUtil {

    /**
     * jdk代理类属性名称
     */
    final static String AOP_JAVA__TARGET_FIELD = "h";

    /**
     * cglib代理类属性名称
     */
    final static String AOP_CGLIB_TARGET_FIELD = "CGLIB$CALLBACK_0";

    /**
     * cglib代理类类名特有标识
     */
    public static final String CGLIB_CLASS_SEPARATOR = "$$";

    /**
     * 检查是不是jdk动态代理类
     *
     * @param bean
     * @return
     */
    public static boolean checkIsJdkProxy(Object bean) {
        if (bean == null) {
            throw new NullPointerException("bean can not be null");
        }
        return Proxy.isProxyClass(bean.getClass());
    }

    /**
     * 检查是不是cglib动态代理类
     *
     * @param bean
     * @return
     */
    public static boolean checkIsCGlibProxy(Object bean) {
        if (bean == null) {
            throw new NullPointerException("bean can not be null");
        }
        String className = bean.getClass().getName();
        return (className != null && className.contains(CGLIB_CLASS_SEPARATOR));
    }

    /**
     * 获取jdk动态代理类的代理目标类
     *
     * @param proxy
     * @return
     * @throws NoSuchFieldException
     */
    public static Class getJdkProxyTarget(Object proxy) throws NoSuchFieldException, IllegalAccessException {
        Field h = proxy.getClass().getSuperclass().getDeclaredField(AOP_JAVA__TARGET_FIELD);
        h.setAccessible(true);
        JdkProxyHandler jdkProxyHandler = (JdkProxyHandler) h.get(proxy);
        return jdkProxyHandler.getTargetClass();
    }

    /**
     * 获取cglib动态代理对象的代理目标类
     *
     * @param proxy
     * @return
     */
    public static Class getCGlibProxyTarget(Object proxy) throws NoSuchFieldException, IllegalAccessException {
        Field h = proxy.getClass().getDeclaredField(AOP_CGLIB_TARGET_FIELD);
        h.setAccessible(true);
        CGlibProxyMethodInterceptor cGlibProxyMethodInterceptor = (CGlibProxyMethodInterceptor) h.get(proxy);
        return cGlibProxyMethodInterceptor.getTargetClass();
    }

    /**
     * Jdk动态代理
     *
     * @param obj 委托对象
     * @return 代理对象
     */
    public static Object getJdkProxy(Object obj, InvocationHandler invocationHandler) {
        // 获取代理对象
        return Proxy.newProxyInstance(obj.getClass().getClassLoader(), obj.getClass().getInterfaces(), invocationHandler);
    }

    /**
     * Jdk动态代理
     *
     * @param clazz
     * @return
     */
    public static <T> T getJdkProxy(Class<T> clazz, InvocationHandler invocationHandler) {
        // 获取代理对象
        return (T) Proxy.newProxyInstance(clazz.getClassLoader(), clazz.getInterfaces(), invocationHandler);
    }

    /**
     * Jdk动态代理接口
     *
     * @param interfaceClazz
     * @return
     */
    public static <T> T getInterfaceJdkProxy(Class<T> interfaceClazz, InvocationHandler invocationHandler) {
        // 获取接口代理对象
        return (T) Proxy.newProxyInstance(interfaceClazz.getClassLoader(), new Class[]{interfaceClazz}, invocationHandler);
    }

    /**
     * 使用cglib动态代理生成代理对象
     *
     * @param obj 委托对象
     * @return
     */
    public static Object getCglibProxy(Object obj, MethodInterceptor methodInterceptor) {
        return Enhancer.create(obj.getClass(), methodInterceptor);
    }

    /**
     * 使用cglib动态代理生成代理对象
     *
     * @param clazz 委托对象
     * @return
     */
    public static <T> T getCglibProxy(Class<T> clazz, MethodInterceptor methodInterceptor) {
        return (T) Enhancer.create(clazz, methodInterceptor);
    }

}
