package com.rubin.spring.context.utils;

import org.reflections.Reflections;
import org.reflections.scanners.FieldAnnotationsScanner;
import org.reflections.scanners.SubTypesScanner;
import org.reflections.scanners.TypeAnnotationsScanner;
import org.reflections.util.ClasspathHelper;
import org.reflections.util.ConfigurationBuilder;

import java.io.File;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 反射工具类
 */
public class ReflectUtil {

    /**
     * 创建实体
     *
     * @param classPath
     * @return
     * @throws ClassNotFoundException
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    public static Object createInstance(String classPath) {
        Object o;
        try {
            Class clazz = Class.forName(classPath);
            o = clazz.newInstance();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            throw new RuntimeException("the class " + classPath + " can not be found");
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            throw new RuntimeException("load class " + classPath + " fail");
        } catch (InstantiationException e) {
            e.printStackTrace();
            throw new RuntimeException("load class " + classPath + " fail");
        }
        return o;
    }

    /**
     * 创建类实例
     *
     * @param clazz
     * @return
     */
    public static Object createInstance(Class clazz) {
        Object o;
        try {
            o = clazz.newInstance();
        } catch (InstantiationException e) {
            e.printStackTrace();
            throw new RuntimeException("load class " + clazz.getCanonicalName() + " fail");
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            throw new RuntimeException("load class " + clazz.getCanonicalName() + " fail");
        }
        return o;
    }

    /**
     * 获取Class类
     *
     * @param classPath
     * @return
     */
    public static Class getClass(String classPath) {
        try {
            return Class.forName(classPath);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            throw new RuntimeException("the class " + classPath + "can not be found");
        }
    }

    /**
     * 注入属性值
     *
     * @param o
     * @param fieldName
     * @param fieldValue
     */
    public static void injectFieldValue(Object o, String fieldName, Object fieldValue) {
        try {
            Field field = o.getClass().getDeclaredField(fieldName);
            field.setAccessible(true);
            field.set(o, fieldValue);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
            throw new RuntimeException("there are no field named " + fieldName + " in class " + o.getClass().getName());
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            throw new RuntimeException("inject field value fail : " + fieldName);
        }
    }

    /**
     * set注入属性值
     *
     * @param o
     * @param fieldName
     * @param fieldValue
     */
    public static void setFieldValue(Object o, String fieldName, Object fieldValue) {
        Method[] declaredMethods = o.getClass().getDeclaredMethods();
        for (int i = 0; i < declaredMethods.length; i++) {
            if (declaredMethods[i].getName().equalsIgnoreCase("set" + fieldName)) {
                try {
                    declaredMethods[i].invoke(o, new Object[]{fieldValue});
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                    throw new RuntimeException("set field value fail : " + fieldName);
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                    throw new RuntimeException("set field value fail : " + fieldName);
                }
            }
        }
    }

    /**
     * 获取接口所有实现类
     *
     * @param interfaceClass
     * @return
     */
    public static Set<Class> getAllSubTypeOf(Class interfaceClass) {
        if (!interfaceClass.isInterface()) {
            throw new RuntimeException("the class must be interface");
        }
        Reflections reflections = new Reflections("com.rubin.spring");
        return reflections.getSubTypesOf(interfaceClass);
    }

    /**
     * 获取某包下面带有某个注解的所有接口
     *
     * @param scanPath
     * @return
     */
    public static Set<Class> getAllInterfaceWithAnnotation(String scanPath, Class annotationClass) {
        if (StringUtil.EMPTY.equals(scanPath)) {
            return new HashSet<>();
        }
        Reflections reflections = new Reflections(new ConfigurationBuilder().setUrls(ClasspathHelper.forPackage(scanPath)).setScanners(new FieldAnnotationsScanner(), new TypeAnnotationsScanner(), new SubTypesScanner(false)));
        Set<Class> allTypes = reflections.getTypesAnnotatedWith(annotationClass);
        if (CollectionUtil.isEmpty(allTypes)) {
            return new HashSet<>();
        }
        Set<Class> collect = allTypes.stream().filter(Class::isInterface).collect(Collectors.toSet());
        return collect;
    }

    /**
     * 获取某包下面所有的带有某注解类的类类型，包括子注解
     *
     * @param scanPath
     * @param annotationClass
     * @return
     */
    public static Set<Class> getAllObjectClassWithAnnotationOrSubAnnotation(String scanPath, Class annotationClass) {
        Set<String> classPaths = new HashSet<>();
        doGetAllObjectClassPaths(classPaths, scanPath);
        Set<Class> classes = classPaths.stream()
                .map(ReflectUtil::getClass)
                .filter(clazz -> ReflectUtil.hasAnnotationOrSubAnnotation(clazz, annotationClass))
                .collect(Collectors.toSet());
        return classes;
    }

    /**
     * 获取该路径下的所有类的路径
     *
     * @param classPaths
     * @param scanPath
     */
    private static void doGetAllObjectClassPaths(Set<String> classPaths, String scanPath) {
        String folderPath = Thread.currentThread().getContextClassLoader().getResource(StringUtil.EMPTY).getPath() + scanPath.replace(".", File.separator);
        File folder = new File(folderPath);
        File[] files = folder.listFiles();
        if (files == null) {
            return;
        }
        for (int i = 0; i < files.length; i++) {
            if (files[i].isDirectory()) {
                doGetAllObjectClassPaths(classPaths, scanPath + "." + files[i].getName());
            } else if (files[i].getName().endsWith(".class")) {
                classPaths.add(scanPath + "." + files[i].getName().replace(".class", StringUtil.EMPTY));
            }
        }
    }

    /**
     * 校验改类上面有没有该注解或者子注解
     *
     * @param clazz
     * @param annotationClass
     * @return
     */
    public static boolean hasAnnotationOrSubAnnotation(Class clazz, Class annotationClass) {
        if (clazz.isAnnotationPresent(annotationClass)) {
            return true;
        }
        Annotation[] annotations = clazz.getAnnotations();
        for (int i = 0; i < annotations.length; i++) {
            if ( annotations[i].annotationType().isAnnotationPresent(annotationClass)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取clazz的接口集合
     *
     * @param clazz
     * @return
     */
    public static List<Class> getInterfaces(Class clazz) {
        List<Class> interfaces = new ArrayList<>();
        Class[] interfaceArr = clazz.getInterfaces();
        if (interfaceArr != null && interfaceArr.length > 0) {
            interfaces = Arrays.asList(interfaceArr);
        }
        return interfaces;
    }

    /**
     * 获取子注解
     *
     * @param clazz
     * @param annotationClass
     * @return
     */
    public static Annotation getSubAnnotation(Class clazz, Class annotationClass) {
        Annotation[] annotations = clazz.getAnnotations();
        for (int i = 0; i < annotations.length; i++) {
            if (annotations[i].annotationType().isAnnotationPresent(annotationClass)) {
                return annotations[i];
            }
        }
        throw new RuntimeException("there are no subType matched the annotation " + annotationClass.getCanonicalName());
    }

}
