package com.rubin.client;

import com.rubin.client.dao.IAccountInfoDao;
import com.rubin.client.pojo.AccountInfo;
import com.rubin.client.service.IAccountService;
import com.rubin.client.service.IAccountTransactionInterfaceService;
import com.rubin.client.service.IAccountTransactionService;
import com.rubin.client.service.impl.AccountMethodTransactionService;
import com.rubin.client.service.impl.AccountTransactionService;
import com.rubin.spring.context.ApplicationContext;
import com.rubin.spring.context.ClassPathXmlApplicationContext;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.UndeclaredThrowableException;
import java.math.BigDecimal;

/**
 * 客户端测试类
 */
public class MyBatisSpringTest {

    ApplicationContext applicationContext;

    @Before
    public void before() {
        applicationContext = new ClassPathXmlApplicationContext("applicationContext.xml");
    }

    /**
     * 测试容器初始化成功
     */
    @Test
    public void applicationContextInitSuccessTest() {
        Assert.assertNotNull(applicationContext);
        Assert.assertNotNull(applicationContext.getBean("dataSource"));
        Assert.assertNotNull(applicationContext.getBean("connectionFactory"));
    }

    /**
     * 测试注入mapper成功 并可以正常调用其方法
     */
    @Test
    public void daoSelectTest() {
        IAccountInfoDao iAccountInfoDao = (IAccountInfoDao) applicationContext.getBean(IAccountInfoDao.class);
        AccountInfo accountInfo = new AccountInfo();
        accountInfo.setCardNo("NO123456789");
        Assert.assertNotNull(iAccountInfoDao.selectByCardNo(accountInfo));
    }

    /**
     * 测试service注入成功 并可以正常调用其方法
     */
    @Test
    public void serviceSelectTest() {
        IAccountService iAccountService = (IAccountService) applicationContext.getBean("accountService");
        Assert.assertNotNull(iAccountService.getByCardNo("NO123456789"));
        iAccountService = (IAccountService) applicationContext.getBean(IAccountService.class);
        Assert.assertNotNull(iAccountService.getByCardNo("NO987654321"));
    }

    /**
     * 测试未添加事务的方法导致数据错乱成功
     */
    @Test(expected = UndeclaredThrowableException.class)
    public void serviceMethodTransferWithoutTransactionTest() {
        IAccountService iAccountService = (IAccountService) applicationContext.getBean("accountService");
        iAccountService.transferWithoutTransaction("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的方法事务回滚成功
     */
    @Test(expected = UndeclaredThrowableException.class)
    public void serviceMethodTransferWithTransactionRollbackTest() {
        IAccountService iAccountService = (IAccountService) applicationContext.getBean("accountService");
        iAccountService.transferWithTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的方法事务正常提交成功
     */
    @Test
    public void serviceMethodTransferWithTransactionSuccessTest() {
        IAccountService iAccountService = (IAccountService) applicationContext.getBean("accountService");
        iAccountService.transferWithTransactionSuccess("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的类事务回滚成功
     */
    @Test(expected = UndeclaredThrowableException.class)
    public void serviceClassTransferWithTransactionRollbackTest() {
        IAccountTransactionService iAccountTransactionService = (IAccountTransactionService) applicationContext.getBean(IAccountTransactionService.class);
        iAccountTransactionService.transferWithTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的类事务正常提交成功
     */
    @Test
    public void serviceClassTransferWithTransactionSuccessTest() {
        IAccountTransactionService iAccountTransactionService = (IAccountTransactionService) applicationContext.getBean(IAccountTransactionService.class);
        iAccountTransactionService.transferWithTransactionSuccess("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的接口事务回滚成功
     */
    @Test(expected = UndeclaredThrowableException.class)
    public void serviceInterfaceTransferWithTransactionRollbackTest() {
        IAccountTransactionInterfaceService iAccountTransactionInterfaceService = (IAccountTransactionInterfaceService) applicationContext.getBean(IAccountTransactionInterfaceService.class);
        iAccountTransactionInterfaceService.transferWithTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试添加事务的接口事务正常提交成功
     */
    @Test
    public void serviceInterfaceTransferWithTransactionSuccessTest() {
        IAccountTransactionInterfaceService iAccountTransactionInterfaceService = (IAccountTransactionInterfaceService) applicationContext.getBean(IAccountTransactionInterfaceService.class);
        iAccountTransactionInterfaceService.transferWithTransactionSuccess("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试CGLIB代理成功
     */
    @Test
    public void cglibProxySuccessTest() {
        AccountTransactionService accountTransactionService = (AccountTransactionService) applicationContext.getBean(AccountTransactionService.class);
        Assert.assertNotNull(accountTransactionService);
    }

    /**
     * 测试CGLIB代理类事务回滚成功
     */
    @Test(expected = InvocationTargetException.class)
    public void serviceCglibTransferWithTransactionRollbackTest() {
        AccountTransactionService accountTransactionService = (AccountTransactionService) applicationContext.getBean(AccountTransactionService.class);
        accountTransactionService.transferWithTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试CGLIB代理类事务正常提交成功
     */
    @Test
    public void serviceCglibTransferWithTransactionSuccessTest() {
        AccountTransactionService accountTransactionService = (AccountTransactionService) applicationContext.getBean(AccountTransactionService.class);
        accountTransactionService.transferWithTransactionSuccess("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试CGLIB代理类未添加事务方法导致数据错乱
     */
    @Test(expected = InvocationTargetException.class)
    public void serviceCglibTransferMethodWithoutTransactionRollbackTest() {
        AccountMethodTransactionService accountMethodTransactionService = (AccountMethodTransactionService) applicationContext.getBean(AccountMethodTransactionService.class);
        accountMethodTransactionService.transferWithoutTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试CGLIB代理类添加事务方法回滚成功
     */
    @Test(expected = InvocationTargetException.class)
    public void serviceCglibTransferMethodWithTransactionRollbackTest() {
        AccountMethodTransactionService accountMethodTransactionService = (AccountMethodTransactionService) applicationContext.getBean(AccountMethodTransactionService.class);
        accountMethodTransactionService.transferWithTransactionRollback("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

    /**
     * 测试CGLIB代理类添加事务方法正常提交成功
     */
    @Test
    public void serviceCglibTransferMethodWithTransactionSuccessTest() {
        AccountMethodTransactionService accountMethodTransactionService = (AccountMethodTransactionService) applicationContext.getBean(AccountMethodTransactionService.class);
        accountMethodTransactionService.transferWithTransactionSuccess("NO123456789", "NO987654321", new BigDecimal(100.00));
    }

}
