package com.rubin.spring.mybatis.handler;

import com.rubin.spring.context.handler.JdkProxyHandler;
import com.rubin.spring.ex.annotation.Transactional;
import com.rubin.spring.ex.manager.TransactionManager;
import com.rubin.spring.mybatis.utils.AnnotationUtil;
import lombok.AllArgsConstructor;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

/**
 * 事务支持代理逻辑类
 */
@AllArgsConstructor
public class TransactionInvocationHandler implements InvocationHandler, JdkProxyHandler {

    /**
     * 事务管理器
     */
    private TransactionManager transactionManager;

    /**
     * 代理类的类类型
     */
    private Class<?> targetClass;

    /**
     * 代理目标类实体
     */
    private Object target;

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if (needTransaction(method.getName())) {
            Object result;
            try {
                transactionManager.start();
                result = method.invoke(target, args);
                transactionManager.commit();
            } catch (Exception e) {
                transactionManager.rollback();
                throw e;
            }
            return result;
        }
        return method.invoke(target, args);
    }

    /**
     * 校验是否需要事务处理
     * 进入代理方法的 都是标记支持事务的
     * 重点查看是不是方法和类以及接口均没有注解的情况
     *
     * @param methodName
     * @return
     */
    private boolean needTransaction(String methodName) {
        Method[] declaredMethods = targetClass.getDeclaredMethods();
        Method proxyMethod = null;
        for (int i = 0; i < declaredMethods.length; i++) {
            if (methodName.equalsIgnoreCase(declaredMethods[i].getName())) {
                proxyMethod = declaredMethods[i];
                break;
            }
        }
        if (proxyMethod == null) {
            throw new RuntimeException("the method" + methodName + " can not find in " + targetClass.getSimpleName());
        }
        boolean result = AnnotationUtil.hasAnnotation(targetClass, Transactional.class) || AnnotationUtil.hasAnnotation(proxyMethod, Transactional.class);
        return result;
    }

    /**
     * 获取代理的目标类
     *
     * @return
     */
    @Override
    public Class<?> getTargetClass() {
        return targetClass;
    }

}
