package com.rubin.spring.mybatis.processor;

import com.rubin.spring.context.factory.AutowiredBeanFactory;
import com.rubin.spring.context.factory.BeanFactory;
import com.rubin.spring.context.processor.BeanPostProcessor;
import com.rubin.spring.context.utils.ProxyUtil;
import com.rubin.spring.ex.annotation.Transactional;
import com.rubin.spring.ex.manager.TransactionManager;
import com.rubin.spring.mybatis.handler.TransactionInvocationHandler;
import com.rubin.spring.mybatis.interceptor.TransactionMethodInterceptor;
import com.rubin.spring.mybatis.utils.AnnotationUtil;

import java.lang.reflect.Method;

/**
 * 事务包装器
 * Created by rubin on 4/4/21.
 */
public class TransactionalBeanPostProcessor implements BeanPostProcessor {

    private AutowiredBeanFactory autowiredBeanFactory;

    /**
     * 将需要事务处理的bean封装
     *
     * @param beanFactory
     * @param beanName
     * @param beanObject
     * @return
     */
    @Override
    public Object postProcessBeforeRegister(BeanFactory beanFactory, String beanName, Object beanObject) {
        if (checkNeedTransaction(beanObject)) {
            Class<?>[] interfaces = beanObject.getClass().getInterfaces();
            TransactionManager transactionManager = (TransactionManager) beanFactory.getBean(TransactionManager.class);
            if (interfaces != null && interfaces.length > 0) {
                beanObject = ProxyUtil.getJdkProxy(beanObject, new TransactionInvocationHandler(transactionManager, beanObject.getClass(), beanObject));
            } else {
                beanObject = ProxyUtil.getCglibProxy(beanObject, new TransactionMethodInterceptor(transactionManager, beanObject.getClass(), beanObject));
            }
        }
        return beanObject;
    }

    /**
     * 校验改bean是否需要事务
     *
     * @param bean
     * @return
     */
    private boolean checkNeedTransaction(Object bean) {
        Class clazz = bean.getClass();
        // 此处会将继承的注解也扫描出来，所以不用区分是不是接口
        if (AnnotationUtil.hasAnnotation(clazz, Transactional.class)) {
            return true;
        }
        if (!clazz.isInterface()) {
            if (checkMethodNeedTransaction(clazz.getDeclaredMethods())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 校验是否有方法需要事务处理
     *
     * @param declaredMethods
     * @return
     */
    private boolean checkMethodNeedTransaction(Method[] declaredMethods) {
        for (int i = 0; i < declaredMethods.length; i++) {
            if (AnnotationUtil.hasAnnotation(declaredMethods[i], Transactional.class)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 执行顺序 升序排列执行
     *
     * @return
     */
    @Override
    public Integer getOrder() {
        return 0;
    }

}
