package com.rubin.spring.mvc.servlet;

import com.alibaba.fastjson.JSON;
import com.rubin.spring.context.AnnotationApplicationContext;
import com.rubin.spring.context.ApplicationContext;
import com.rubin.spring.context.ClassPathXmlApplicationContext;
import com.rubin.spring.context.utils.CollectionUtil;
import com.rubin.spring.context.utils.ReflectUtil;
import com.rubin.spring.context.utils.StringUtil;
import com.rubin.spring.mvc.Handler;
import com.rubin.spring.mvc.annotation.RequestMapping;
import com.rubin.spring.mvc.annotation.RestController;
import com.rubin.spring.mvc.annotation.Security;
import com.rubin.spring.mvc.scanner.ControllerScanner;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 处理器映射器
 * Created by rubin on 4/4/21.
 */
public class DispatcherServlet extends HttpServlet {

    private ApplicationContext applicationContext;

    private List<Handler> handlerMapping = new ArrayList<>();

    private static final String SLASHES_REGEX_STR = "(^/+)|(/+$)";

    public DispatcherServlet() {

    }

    public DispatcherServlet(ApplicationContext applicationContext) {
        this.applicationContext = applicationContext;
    }

    @Override
    public void init(ServletConfig config) throws ServletException {
        if (this.applicationContext == null) {
            String contextConfigLocation = config.getInitParameter("contextConfigLocation");
            this.applicationContext = new ClassPathXmlApplicationContext(contextConfigLocation);
        }
        initHandlers();
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        Handler handler = getHandler(req);
        if (handler == null) {
            resp.getWriter().write("404 not found");
            return;
        }
        // 权限校验
        if (!checkSecurity(handler, req)) {
            try {
                resp.getWriter().write("you have no permission to pass");
                return;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        // 处理请求
        handleRequest(handler, req, resp);
    }

    /**
     * 权限校验
     *
     * @param handler
     * @param req
     */
    private boolean checkSecurity(Handler handler, HttpServletRequest req) {
        Object restControllerInstance = applicationContext.getBean(handler.getControllerClass());
        Method method = handler.getMethod();
        if (!method.isAnnotationPresent(Security.class) && !restControllerInstance.getClass().isAnnotationPresent(Security.class)) {
            return true;
        }
        Security securityAnnotation;
        if (method.isAnnotationPresent(Security.class)) {
            securityAnnotation = method.getAnnotation(Security.class);
        } else {
            securityAnnotation = restControllerInstance.getClass().getAnnotation(Security.class);
        }
        String[] usernames = securityAnnotation.values();
        if (usernames.length == 0) {
            return false;
        }
        String usernameParam = req.getParameter("username");
        if (StringUtil.isBlank(usernameParam)) {
            return false;
        }
        for (int i = 0; i < usernames.length; i++) {
            if (usernames[i].equals(usernameParam)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 处理请求
     *
     * @param handler
     * @param req
     * @param resp
     */
    private void handleRequest(Handler handler, HttpServletRequest req, HttpServletResponse resp) {
        // 参数绑定
        Class<?>[] parameterTypes = handler.getMethod().getParameterTypes();

        // 根据上述数组长度创建一个新的数组（参数数组，是要传入反射调用的）
        Object[] paraValues = new Object[parameterTypes.length];

        // 以下就是为了向参数数组中塞值，而且还得保证参数的顺序和方法中形参顺序一致
        Map<String, String[]> parameterMap = req.getParameterMap();

        // 遍历request中所有参数
        for (Map.Entry<String, String[]> param : parameterMap.entrySet()) {
            String value = StringUtils.join(param.getValue(), ",");

            // 如果参数和方法中的参数匹配上了，填充数据
            if (!handler.getParamIndexMapping().containsKey(param.getKey())) {
                continue;
            }

            // 方法形参确实有该参数，找到它的索引位置，对应的把参数值放入paraValues
            Integer index = handler.getParamIndexMapping().get(param.getKey());

            paraValues[index] = value;
        }

        Integer requestIndex = handler.getParamIndexMapping().get(HttpServletRequest.class.getSimpleName());
        if (requestIndex != null) {
            paraValues[requestIndex] = req;

        }
        Integer responseIndex = handler.getParamIndexMapping().get(HttpServletResponse.class.getSimpleName());
        if (responseIndex != null) {
            paraValues[responseIndex] = resp;

        }

        // 最终调用handler的method属性
        try {
            Object result = handler.getMethod().invoke(applicationContext.getBean(handler.getControllerClass()), paraValues);
            if (result != null) {
                resp.setContentType("application/json;charset=UTF-8");
                resp.getWriter().write(JSON.toJSONString(result));
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取对应的处理器
     *
     * @param req
     * @return
     */
    private Handler getHandler(HttpServletRequest req) {
        if (handlerMapping.isEmpty()) {
            return null;
        }

        String url = req.getRequestURI();

        for (Handler handler : handlerMapping) {
            Matcher matcher = handler.getPattern().matcher(url);
            if (!matcher.matches()) {
                continue;
            }
            return handler;
        }
        return null;
    }

    /**
     * 初始化所有映射器
     */
    private void initHandlers() {
        String basePackage = "/";
        if (this.applicationContext instanceof ClassPathXmlApplicationContext) {
            ControllerScanner controllerScanner = (ControllerScanner) applicationContext.getBean(ControllerScanner.class);
            basePackage = controllerScanner.getBasePackage();
        } else if (this.applicationContext instanceof AnnotationApplicationContext) {
            basePackage = ((AnnotationApplicationContext) this.applicationContext).getAnnotationScanBasePackage();
        }
        Set<Class> restControllerClasses = ReflectUtil.getAllObjectClassWithAnnotationOrSubAnnotation(basePackage, RestController.class);
        if (CollectionUtil.isEmpty(restControllerClasses)) {
            return;
        }
        restControllerClasses.stream().forEach(restControllerClass -> initHandler(restControllerClass));
    }

    /**
     * 初始化映射器并加入映射器列表
     *
     * @param restControllerClass
     */
    private void initHandler(Class<?> restControllerClass) {
        String baseUrl = StringUtil.EMPTY;
        if (restControllerClass.isAnnotationPresent(RequestMapping.class)) {
            RequestMapping annotation = restControllerClass.getAnnotation(RequestMapping.class);
            baseUrl = handleUrlSlash(annotation.value());
        }
        Method[] declaredMethods = restControllerClass.getDeclaredMethods();
        for (int i = 0; i < declaredMethods.length; i++) {
            Method method = declaredMethods[i];
            if (!method.isAnnotationPresent(RequestMapping.class)) {
                continue;
            }
            String childUrl = method.getAnnotation(RequestMapping.class).value();
            String url = "";
            if (StringUtil.isNotBlank(baseUrl)) {
                url = "/" + baseUrl + "/" + handleUrlSlash(childUrl);
            } else {
                url = "/" + handleUrlSlash(childUrl);
            }
            Handler handler = new Handler();
            handler.setControllerClass(restControllerClass);
            handler.setMethod(method);
            handler.setPattern(Pattern.compile(url));
            handler.setParamIndexMapping(new ConcurrentHashMap<>());
            // 计算方法的参数位置信息
            Parameter[] parameters = method.getParameters();
            for (int j = 0; j < parameters.length; j++) {
                Parameter parameter = parameters[j];
                if (parameter.getType().equals(HttpServletRequest.class) || parameter.getType().equals(HttpServletResponse.class)) {
                    // 如果是request和response对象，那么参数名称写HttpServletRequest和HttpServletResponse
                    handler.getParamIndexMapping().put(parameter.getType().getSimpleName(), j);
                } else {
                    handler.getParamIndexMapping().put(parameter.getName(), j);
                }

            }
            handlerMapping.add(handler);
        }
    }

    /**
     * 将路径中起始和最终的斜杠（/）去掉，防止后面字符串拼接过程中出现多个斜杠
     *
     * @param url
     * @return
     */
    private String handleUrlSlash(String url) {
        return url.replaceAll(SLASHES_REGEX_STR, StringUtil.EMPTY);
    }

}
